#!/usr/bin/python
# -*- coding: utf-8 -*-

""" The `Roles` module provides the core system for creating and using
the `Roles` system with KIM.

"""


class BaseRole(object):
    """Role Abstract Base Class.  All `Role` types MUST inherit from this
    base class.
    """
    pass


class Role(BaseRole):
    """:class:`kim.roles.Role` is a way to affect the `Types`
    used in a Mapping or Mapped Serializer.  They allow users
    to specify custom sets of field names that are permitted
    or disabled for a defined mapping.

    :param name: The name given to this Role e.g::
        Role('my_role')

    :param field_names: The field names for this role.  Any number of field
    names may be specified as positional arguments
    to the :class:`Role` constructor e.g::
        Role('my_role', 'field_1', 'field_2)

    :param whitelist: Specify wether the `field_names` defined should be allowed
    or disabled for this role. (see :meth:`membership`)
    """

    def __init__(self, *args, **kwargs):
        self.field_names = args
        self.whitelist = kwargs.pop('whitelist', True)

    def membership(self, field_name):
        """Returns a boolean indicating the membership for `field_name`
        in this `role`

        For whitelisted Roles, the :meth:`membership` will check to see
        if `field_name` is defined in `self.field_names`. For blacklisted
        (whitelist=False) the inverse is checked

        This method may be overriden to provide more complex membership
        tests if required.

        e.g::
            >>>role = Role('my_role', 'name', 'email')
            >>>role.membership('foo')
            >>>False

        :rtype: bool
        :returns: True if the field_name is permitted in this role.
        """
        if self.whitelist:
            return field_name in self.field_names
        else:
            return field_name not in self.field_names

    def get_mapping(self, mapping):
        """Creates a new mapping based on this role.

        :param mapping: a :class:`kim.mapping.BaseMapping` mapping type.

        :rtype: :class:`kim.mapping.BaseMapping`
        :returns: a new mapping from this role

        .. seealso::
            :func:`create_mapping_from_role`
        """

        return create_mapping_from_role(self, mapping)


def create_mapping_from_role(role, mapping):
    """utility method that creates a new mapping from `mapping`
    pulling in the allowed fields defined in `role.field_names`

    :param role: :class:`BaseRole` type
    :param mapping: :class:`kim.mapping.BaseMapping` type

    :rtype: :class:`kim.mapping.BaseMapping`
    :returns: New mapping constructred from role.field_names
    """
    fields = [field for field in mapping.fields
              if role.membership(field.field_id)]

    # We create a new class here from mapping to allow custom mappers
    # TODO what other args, kwargs can a mapping have?
    MappingKlass = mapping.__class__
    return MappingKlass(
        *fields,
        validator=mapping.validator
    )


def _create_role(fields, role_base=None, whitelist=True, **kwargs):
    """Factory function for generating new Roles derived from an
    optional custom role class

    :param name: the name given to this role
    :param fields: iterable of field names
    :param role_base: specify a custom role type to create a role from
    :param whitelist: specify the role type

    .. seealso::
        :class:`Role`

    :returns: New :class:`Role` type
    """

    BaseKlass = role_base or Role
    return BaseKlass(
        whitelist=whitelist,
        *fields,
        **kwargs
    )


def whitelist(*fields, **kwargs):
    """Helper function that explicitly creates a new :class`Role` type
    setting the whitelist option to True

    :param fields: iterable of field names
    :param role_base: specify a custom role type to create a role from

    e.g::
        role=whitelist('name', 'email')

    .. seealso::
        :func:`_create_role`

    :returns: New :class:`Role` type
    """
    role_base = kwargs.pop('role_base', None)
    return _create_role(fields,
                        role_base=role_base,
                        whitelist=True,
                        **kwargs)


def blacklist(*fields, **kwargs):
    """Helper function that explicitly creates a new :class`Role` type
    setting the whitelist option to False

    :param name: the name given to this role
    :param fields: iterable of field names
    :param role_base: specify a custom role type to create a role from

    e.g::
        role=blacklist('user_public', ['name', 'email'])

    .. seealso::
        :func:`_create_role`

    :returns: New :class:`Role` type
    """
    role_base = kwargs.pop('role_base', None)
    return _create_role(fields,
                        role_base=role_base,
                        whitelist=False,
                        **kwargs)



from inspect import isclass
from collections import OrderedDict
import json
from functools import partial

from .exceptions import RoleNotFound, ConfigurationError
from .mapping import Mapping, serialize, marshal
from .utils import is_role
from .types import BaseType
from .fields import Field


class SerializerMetaclass(type):

    def __new__(mcs, name, bases, attrs):

        current_fields = []
        current_validators = []
        for key, value in list(attrs.items()):
            if isinstance(value, Field):
                # Normal Field attributes
                current_fields.append((key, value))
                attrs.pop(key)
            elif callable(value) and key.startswith('validate_'):
                # validate_X callables
                validation_target = key[9:] # stripe 'validate_' from start
                current_validators.append((validation_target, value))
                attrs.pop(key)
            elif isinstance(value, BaseType):
                # Handle common mistake of failing to wrap types in Field()
                # where passed as instance
                type_name = value.__class__.__name__
                raise ConfigurationError('Serializer attributes must be wrapped ' \
                    'in Field. Raw type %s found on %s. Did you mean Field(%s())?' %
                    (type_name, name, type_name))
            elif isclass(value) and issubclass(value, BaseType):
                # Handle common mistake of failing to wrap types in Field()
                # where passed as class
                type_name = value.__name__
                raise ConfigurationError('Serializer attributes must be wrapped ' \
                    'in Field. Raw type %s found on %s. Did you mean Field(%s)?' %
                    (type_name, name, type_name))

        attrs['declared_fields'] = OrderedDict(current_fields)
        attrs['declared_validators'] = OrderedDict(current_validators)

        new_class = (super(SerializerMetaclass, mcs)
                     .__new__(mcs, name, bases, attrs))

        # Walk through the MRO.
        declared_fields = OrderedDict()
        declared_validators = OrderedDict()
        for base in reversed(new_class.__mro__):
            # Collect fields from base class.
            if hasattr(base, 'declared_fields'):
                declared_fields.update(base.declared_fields)
            # Collect fields from base class.
            if hasattr(base, 'declared_validators'):
                declared_validators.update(base.declared_validators)

        new_class.base_fields = declared_fields
        new_class.declared_fields = declared_fields
        new_class.base_validators = declared_validators
        new_class.declared_validators = declared_validators

        return new_class


class SerializerOpts(object):

    def __init__(self, meta):

        self.roles = getattr(meta, 'roles', {})


class BaseSerializer(object):

    class Meta:
        """meta class for providing extra options for a serializer

        Options::

            Roles may be assigned to a serializer using the roles
            meta option.  roles should be specified as a dict of
            role name: Role() instance.

            roles = {'public': whitelist('field_a', 'field_b')}

            .. seealso::
                :class:`kim.roles.Role`

        """


class Serializer(BaseSerializer):
    """:class:`kim.serializer.Serializer` is a declarative wrapper for
    generating :class:`kim.mapping.Mapping`s. It also provides convinience
    methods for marshalling data against it's mapping.

    Whilst it is not nessasary to use a :class:`kim.serializer.Serializer` to
    use Kim, it is recommended for most users as the default way to
    interact with the low level :class:`kim.mapping.Mapping` API.

    Serializers consist of attributes representing fields. These will become
    keys in the resulting serialized data. If the Field does not list a source,
    the source will default to the field name.

    e.g.::
        class MySerializer(Serializer):
            name = Field(String)
            age = Field(Interger, source='user_age')


    .. seealso::

        :class:`kim.serializers.Field`

    """

    __metaclass__ = SerializerMetaclass

    def __init__(self, data=None, input=None):
        self.opts = SerializerOpts(self.Meta)
        self.__mapping__, self.fields = self._build_mapping()

    def _build_mapping(self):
        top_level_validator = getattr(self, 'validate', None)
        mapping = Mapping(validator=top_level_validator)
        fields = {}
        for name, field in self.declared_fields.items():
            validator = self.declared_validators.get(name)
            if validator:
                # As the validator will not be called as Serializer.validate_X(),
                # but rather just validate_X(), self will not be passed. Therefore
                # we need to curry it.
                curried_validator = partial(validator, self)
                validators = [curried_validator]
            else:
                validators = []

            if field.name is None:
                field.name = name

            if field._field_id is None:
                field._field_id = name

            field.extra_validators.extend(validators)
            mapping.add_field(field)
            fields[name] = field

        return mapping, fields

    def get_role(self, role):
        """Find and return a serializer role.  `role` may be provided to
        :meth:`serialize` or :meth:`marshal` as a role instance or as
        a string representing a role name defined in `self.opts.roles`

        :raises: RoleNotFound
        :returns: an instance of a role
        """
        if is_role(role):
            return role

        try:
            return self.opts.roles[role]
        except KeyError:
            raise RoleNotFound('Missing role %s' % role)

    def get_mapping(self, role=None):
        """return the serializers mapping.  If `role` is supplied
        then the mapping will be returned via :meth:`get_mapping` method
        on the `role` instance.

        :param role: name of role or role instance.

        :returns: mapping or mapping for a role
        """
        if role:
            role = self.get_role(role)
            return role.get_mapping(self.__mapping__)

        return self.__mapping__

    def serialize(self, data, role=None, **kwargs):
        return serialize(self.get_mapping(role=role), data, **kwargs)

    def json(self, *args, **kwargs):
        return json.dumps(self.serialize(*args, **kwargs))

    def marshal(self, data, role=None, **kwargs):
        return marshal(self.get_mapping(role=role), data, **kwargs)



# encoding: utf-8
from collections import defaultdict

from sqlalchemy.inspection import inspect
from sqlalchemy.orm.exc import NoResultFound

from ..serializers import Serializer
from ..mapping import MarshalVisitor, SerializeVisitor, get_attribute

from ..types import Nested, NumericType, Collection
from ..exceptions import ValidationError, ConfigurationError
from ..utils import recursive_defaultdict


class NestedForeignKey(Nested):
    """Field which can function as a normal Nested field, but can also take an
    integer foreign key in an 'id' field when marshalled. In which case, it
    will look up the required object via self.getter."""

    __visit_name__ = "nested_foreign_key"

    def __init__(self, *args, **kwargs):
        self.getter = kwargs.pop('getter', None)
        self.id_field_name = kwargs.pop('id_field_name', 'id')
        self.allow_updates = kwargs.pop('allow_updates', False)
        self.allow_updates_in_place = kwargs.pop('allow_updates_in_place', False)
        self.allow_create = kwargs.pop('allow_create', False)
        self.remote_class = kwargs.pop('remote_class', None)
        if self.allow_updates_in_place and self.allow_create:
            raise ConfigurationError('allow_create and allow_updates_in_place may not be used together')
        super(NestedForeignKey, self).__init__(*args, **kwargs)

    def valid_id(self, val):
        return any([isinstance(val, int),
                    isinstance(val, basestring) and val.isdigit()])

    def get_object(self, source_value):
        if self.allow_updates_in_place:
            return
        if type(source_value) != dict:
            raise ValidationError('invalid type')
        id = source_value.get(self.id_field_name)
        if self.valid_id(id):
            id = int(id)
            try:
                obj = self.getter(id)
            except NoResultFound:
                obj = None
            if not obj:
                raise ValidationError('invalid id')
            return obj

    def validate(self, source_value):
        if source_value is not None:
            if self.get_object(source_value):
                return True
            else:
                super(NestedForeignKey, self).validate(source_value)


class IntegerForeignKey(NumericType):
    """Field representing an Integer ForeignKey. Behaves as NumericType, but will
    look up the required object via self.getter to ensure it exists/is valid"""

    def __init__(self, *args, **kwargs):
        self.getter = kwargs.pop('getter', None)
        super(IntegerForeignKey, self).__init__(*args, **kwargs)

    def validate(self, source_value):
        super(IntegerForeignKey, self).validate(source_value)
        if source_value is not None:
            try:
                obj = self.getter(source_value)
            except NoResultFound:
                obj = None
            if not obj:
                raise ValidationError('invalid id')
        return True


class RelationshipCollection(Collection):
    __visit_name__ = 'relationship_collection'

    def __init__(self, *args, **kwargs):
        self.remote_class = kwargs.pop('remote_class', None)
        super(RelationshipCollection, self).__init__(*args, **kwargs)


class SQASerializeVisitor(SerializeVisitor):
    def visit_type_nested_foreign_key(self, type, data, **kwargs):
        return self.visit_type_nested(type, data, **kwargs)

    def visit_type_relationship_collection(self, type, data, **kwargs):
        return self.visit_type_collection(type, data)


class SQAMarshalVisitor(MarshalVisitor):
    def __init__(self, *args, **kwargs):
        self.model = kwargs.pop('model', None)
        self.instance = kwargs.pop('instance', None)
        self.partial = kwargs.pop('partial', False)
        super(SQAMarshalVisitor, self).__init__(*args, **kwargs)

    def get_data(self, field):
        """ Override super so if instance and partial we can populate the
        field data from the instance.
        """

        if self.instance and self.partial:
            if field.name in self.data:
                data = get_attribute(self.data, field.name)
            else:
                data = get_attribute(self.instance, field.name)
            return data
        else:
            return super(SQAMarshalVisitor, self).get_data(field)

    def update_output(self, field, value):
        if not field.read_only:
            components = field.source.split('.')
            output = self.output
            for component in components[:-1]:
                output = getattr(output, component)
            setattr(output, components[-1], value)

    def initialise_output(self):
        if not self.instance:
            self.output = self.model()
        else:
            self.output = self.instance

    def _get_relationship_model(self, field):
        if field.field_type.remote_class:
            return field.field_type.remote_class
        # Find what sort of model we require by introspection of
        # the relationship
        components = field.source.split('.')
        RemoteClass = self.output
        for component in components:
            inspection = inspect(RemoteClass)
            relationship = inspection.mapper.relationships.get(component)
            RemoteClass = relationship.mapper.class_
        return RemoteClass

    def visit_type_nested_foreign_key(self, type, data, instance=None, model=None, **kwargs):
        # 1. User has passed an id and no updates are allowed.
        #    Resolve the id to an object and return immediately
        # 2. User has passed an id and updates are allowed.
        #    Resolve the id to an object and call recursively to update it
        # 3. Object already exists, user has not passed an id and in place
        #    updates are allowed. Call recursively to update existing object.
        # 4. User has not passed an id and creation of new objects is allowed
        #    Call recursively to create a new object
        # 5. User has not passed an id and creation of new objects is not
        #    allowed, nor are in place updates. Raise an exception.

        resolved = type.get_object(data)

        if resolved:
            if type.allow_updates:
                return self.Cls(type.get_mapping(), data, instance=resolved, model=model)._run()
            else:
                return resolved
        else:
            if type.allow_updates_in_place:
                return self.Cls(type.get_mapping(), data, instance=instance, model=model)._run()
            elif type.allow_create:
                return self.Cls(type.get_mapping(), data, model=model)._run()
            else:
                raise ValidationError('No id passed and creation or update in place not allowed')

    def visit_field_nested_foreign_key(self, field, data):
        if data is not None:
            existing = get_attribute(self.output, field.source)
            RemoteClass = self._get_relationship_model(field)
            return self.visit_type_nested_foreign_key(field.field_type, data, instance=existing, model=RemoteClass)

    def visit_field_relationship_collection(self, field, data):
        existing_list = get_attribute(self.output, field.source)
        RemoteClass = self._get_relationship_model(field)
        return self.visit_type_relationship_collection(field.field_type, data, instance_list=existing_list, model=RemoteClass)

    def visit_type_relationship_collection(self, type, data, instance_list=None, model=None, **kwargs):
        result = []
        for i, value in enumerate(type.marshal_members(data)):
            try:
                instance = instance_list[i]
            except IndexError:
                instance = None
            value = self.visit_type(type.inner_type, value, instance=instance, model=model)
            result.append(value)
        return result


class SQARawSerializeVisitor(SQASerializeVisitor):
    Cls = SQASerializeVisitor # somewhat hacky: make nested calls go to the parent class not this one

    def _remove_none(self, output):
        # If all components of a dictionary are none, remove it and mark the entire thing as none
        all_none = True
        for k, v in output.iteritems():
            if type(v) == defaultdict:
                output[k] = self._remove_none(v)
            else:
                if v is not None:
                    all_none = False
        if all_none:
            return None
        else:
            return output

    def transform_data(self, data):
        """Transform flat data from a KeyedTuple with keys in the form:
            [
                'id',
                'name',
                'contact_details__phone',
                'contact_details__address__postcode'
            ]
            Into:
            {
                'id': x,
                'name': x,
                'contact_details': {
                    'phone': x,
                    'address': {
                        'postcode': x
                    }
                }
            }
        """
        output = recursive_defaultdict()
        for key in data.keys():
            path = key.split('__')
            target = output
            # Walk along the path until we find the final dict to insert the
            # data into
            for component in path[:-1]:
                target = target[component]
            # And finally set the key on the final dict to the relevant data
            target[path[-1]] = getattr(data, key)

        return self._remove_none(output)

    def __init__(self, mapping, data):
        transformed_data = self.transform_data(data)
        super(SQARawSerializeVisitor, self).__init__(mapping, transformed_data)


class SQASerializer(Serializer):

    def marshal(self, data, instance=None, role=None, **kwargs):
        return SQAMarshalVisitor.run(self.get_mapping(role=role), data, model=self.__model__, instance=instance, **kwargs)

    def serialize(self, data, role=None, raw=False, **kwargs):
        if raw:
            return SQARawSerializeVisitor.run(self.get_mapping(role=role), data, **kwargs)
        else:
            return SQASerializeVisitor.run(self.get_mapping(role=role), data, **kwargs)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from .roles import BaseRole

from collections import defaultdict


def is_valid_type(type_):
    """Validate that `type_` is an instance or subclass of
    :class:`kim.types.BaseType`.

    :param type_: an instance or subclass of :class:`kim.types.BaseType`

    :rtype: boolean
    :returns: True or False
    """
    from kim.types import BaseType

    return isinstance(type_, BaseType)


def is_role(role):
    """evaluate `role` and check wether its a valid role instance or
    valid role subclass.

    :rtype: boolean
    :returns: True or False
    """

    return isinstance(role, BaseRole)


def is_valid_field(field):
    from .fields import Field

    return isinstance(field, Field)


def recursive_defaultdict():
    return defaultdict(recursive_defaultdict)




class KimError(Exception):
    pass


class ValidationError(KimError):
    """Exception used by Types to raise an error when their validate
    method fails

    ..seealso::
        :meth:`kim.types.BaseType.validate`

    """
    pass


class FieldError(ValidationError):
    """A type of :class:`.ValidationError` that also accepts the field
    name specified as `key`

    """

    def __init__(self, key, *args, **kwargs):
        super(FieldError, self).__init__(*args, **kwargs)

        self.key = key


class MappingErrors(KimError):
    pass


class ConfigurationError(KimError):
    pass


class RoleNotFound(KimError):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys

from collections import defaultdict

from .exceptions import ValidationError, MappingErrors, FieldError, KimError
from .utils import is_valid_field


class BaseMapping(object):
    pass


class Mapping(BaseMapping):
    """:class:`kim.mapping.Mapping` is a factory for generating data
    structures in KIM.

    A mapping consists of a collection of kim `field` types.

    `Mappings` are created by passing `Fields` to the
    contructor of :class:`kim.mapping.Mapping`

     e.g.::

        my_mapping = Mapping(
             String('name'),
             Integer('id'),
        )

    The first argument to the Mapping type is the name of this mapping,
    the following arguments may be any mixture of `Field` types.

    :param fields: contains the `collection` of Field types provided

        Any field inherting from :class:`kim.fields.BaseType` is considered
        to be a valid field passed into a mapping.

    :param collection: Provided as a keyword arg to a `mapping` sets the data
                       structure used to store `fields` in. (default list)

    .. seealso::

        :class:`kim.fields.BaseType`

    """

    def __init__(self, *args, **kwargs):
        """:class:`kim.mapping.Mapping` constructor.

        """
        mapped_types = args[0:]

        self.fields = kwargs.get('collection', list())
        self.validator = kwargs.get('validator', None)

        self._arg_loop(mapped_types)

    def __iter__(self):
        return iter(self.fields)

    def _arg_loop(self, items):
        """Iterates over collection of constructor args and assigns
        accordingly.

        :param items: collection of Field and Role type args.

        :returns: None
        """

        for item in items:
            if is_valid_field(item):
                self.add_field(item)

    def add_field(self, field):
        """Add a `field` type to the `fields` collection.

        :param field: A field type

        .. seealso::
            :class:`kim.fields.BaseType`

        :returns: None
        """
        self.fields.append(field)


def _get_attribute(data, attr):
    if attr == '__self__':
        return data
    elif isinstance(data, dict):
        return data.get(attr)
    else:
        return getattr(data, attr, None)


def get_attribute(data, attr):
    """Attempt to find the value for a `field` from `data`.

    :param data: dict like object containing input data
    :param field: mapped field.

    :returns: the value for `field` from `data`
    """
    # Dot notation can be used to span relationships. To handle this, we need
    # to call _get_attribute recursively until we get down to the level
    # specified
    components = attr.split('.')
    for component in components:
        data = _get_attribute(data, component)
    return data


class Visitor(object):
    def __init__(self, mapping, data):
        self.mapping = mapping
        self.data = data
        self.initialise_output()

    @property
    def Cls(self):
        return self.__class__

    def initialise_output(self):
        self.output = {}

    def visit_field(self, field, data):
        name = 'visit_field_%s' % field.field_type.__visit_name__
        func = getattr(self, name, None)
        if func:
            return func(field, data)
        else:
            return self.visit_type(field.field_type, data)

    def visit_type(self, type, data, **kwargs):
        name = 'visit_type_%s' % type.__visit_name__
        return getattr(self, name)(type, data, **kwargs)

    def validate(self, field, data):
        return True

    def _run(self):
       for field in self.mapping:
            data = self.get_data(field)
            if data is None:
                data = field.default
            try:
                if self.validate(field, data):
                    result = self.visit_field(field, data)
                    self.update_output(field, result)
            except ValidationError as e:
                self.errors[field.name].append(e.message)
            except Exception as e:
                # If we catch anything else something's genuinely gone wrong,
                # but the stacktrace will be indecipherable. Append the actual
                # details of which field we're on to help.
                msg = 'Caught error whilst processing %s in %s.\n' \
                      'Original exception was "%s: %s"' % (
                          field.field_id, self.mapping, e.__class__.__name__, e)
                raise KimError(msg), None, sys.exc_info()[2]

       return self.output

    def get_data(self, field):
        raise NotImplementedError

    def update_output(self, field, result):
        raise NotImplementedError

    @classmethod
    def run(cls, mapping, data, many=False, **kwargs):
        if many:
            result = []
            errors = []
            has_errors = False
            for d in data:
                try:
                    result.append(cls(mapping, d, **kwargs)._run())
                    errors.append({})
                except MappingErrors as e:
                    errors.append(e.message)
                    has_errors = True
            if has_errors:
                raise MappingErrors(errors)
            else:
                return result
        else:
            return cls(mapping, data, **kwargs)._run()


class SerializeVisitor(Visitor):
    def get_data(self, field):
        return get_attribute(self.data, field.source)

    def update_output(self, field, result):
        self.output[field.name] = result

    def visit_type_collection(self, type, data, **kwargs):
        result = []
        for value in type.serialize_members(data):
            value = self.visit_type(type.inner_type, value, **kwargs)
            result.append(value)
        return result

    def visit_type_default(self, type, data, **kwargs):
         return type.serialize_value(data)

    def visit_type_nested(self, type, data, **kwargs):
        if data is not None:
            return self.Cls(type.get_mapping(), data)._run()



class MarshalVisitor(Visitor):
    def __init__(self, *args, **kwargs):
        super(MarshalVisitor, self).__init__(*args, **kwargs)
        self.errors = defaultdict(list)

    def get_data(self, field):
        return get_attribute(self.data, field.name)

    def validate(self, field, data):
        if field.read_only:
            return False
        if field.is_valid(data):
            return True

    def update_output(self, field, value):
        if not field.read_only:
            if field.source == '__self__':
                self.output.update(value)
            else:
                # Sources can be specified using dot notation which indicates
                # nested dicts should be created.
                # To handle this, we need to split off all but the last
                # part of the source string (the part after the final dot),
                # and create dicts for all the levels below that if they don't
                # already exist.
                # Finally, now we've resolved the nested level we actually want
                # to update, set the key in the last part to the value passed.
                components = field.source.split('.')
                components_except_last = components[:-1]
                last_component = components[-1]
                current_component = self.output
                for component in components_except_last:
                    current_component.setdefault(component, {})
                    current_component = current_component[component]
                current_component[last_component] = value

    def visit_type_collection(self, type, data, **kwargs):
        result = []
        if data is not None:
            for value in type.marshal_members(data):
                value = self.visit_type(type.inner_type, value, **kwargs)
                result.append(value)
        return result

    def visit_type_default(self, type, data, **kwargs):
        if data is not None:
            return type.marshal_value(data)

    def visit_type_nested(self, type, data, **kwargs):
        if data is not None:
            return MarshalVisitor(type.get_mapping(), data)._run()

    def _run(self):
        output = super(MarshalVisitor, self)._run()
        self.post_process()
        if self.errors:
            raise MappingErrors(self.errors)
        else:
            return output

    def post_process(self):
        if self.mapping.validator:
            try:
                self.mapping.validator(self.output)
            except MappingErrors as e:
                self.errors = e.message



def serialize(mapping, data, many=False):
    return SerializeVisitor.run(mapping, data, many=many)

def marshal(mapping, data, many=False):
    return MarshalVisitor.run(mapping, data, many=many)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Kim
-----------

A framework agnostic serialization and marshaling library written in python.
"""

import os
import sys

from setuptools import setup, find_packages


# Get current directory where setup is running
try:
    SETUP_DIRNAME = os.path.dirname(__file__)
except NameError:
    SETUP_DIRNAME = os.path.dirname(sys.argv[0])

# Change directory
if SETUP_DIRNAME != '':
    os.chdir(SETUP_DIRNAME)

# Paths to requirement files
INSTALL_DEPS = os.path.join('dependencies', 'install.txt')
TEST_DEPS = os.path.join('dependencies', 'test.txt')
DEV_DEPS = os.path.join('dependencies', 'dev.txt')


from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here. outside, the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


def read_dependencies(filename):
    """
    Read requirements file and process them into a list
    fpr usage in the setup function

    :param filename: Path to the file to read line by line
    :type filename: string

    :returns: list -- list of requirements
    """

    dependencies = []
    with open(filename) as f:
        for line in f.readlines():
            if not line or line.startswith('#'):
                continue
            dependencies.append(line.strip())
    return dependencies


def read(name):
    """
    Read file in local current working directory and return the contents

    :param name: The name of the file
    :type name: string

    :returns: string -- Contents of file
    """

    return open(name).read()


setup(
    name='py-kim',
    version=open('VERSION').read().strip(),
    author='Mikey Waites, Jack Saunders',
    author_email='code@bruv.io',
    url='https://github.com/mikeywaites/kim',
    download_url='https://github.com/mikeywaites/kim/releases/tag/0.0.16',
    description='A framework agnostic serialization and marshaling '
                'library written in python.',
    long_description=read('README.rst'),
    packages=find_packages(
        exclude=["tests"]
    ),
    include_package_data=True,
    zip_safe=False,
    install_requires=read_dependencies(INSTALL_DEPS),
    extras_require={
        'develop': read_dependencies(DEV_DEPS)+read_dependencies(TEST_DEPS)
    },
    tests_require=read_dependencies(TEST_DEPS),
    cmdclass={
        'test': PyTest
    },
    dependency_links=[],
    entry_points={},
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content']
    )



import inspect

from .exceptions import ValidationError


class Field(object):
    """Represents a mapping of a field name and `Type` in a `Mapping`

    """

    def __init__(self, *args, **kwargs):
        """Construct a new :py:class:`.Field` object

        :param name: The name of this :py:class:`.Field` inside of a
            :class:`kim.mapping.Mapping` data structure.

            The name arg may be omitted at contsruction time.  This is to
            allow the lower level :py:class:`.Field` object, which maybe used directly
            with the mapping api, to work with the higher level serializer api
            where the name will be set by the
            :py:class:`kim.serializers.Serializer` meta class. e.g::

                Field('id', String())
                Field(String())

        :param field_type: The fields type.  Set using a subclass of
            :py:class:`kim.types.BaseType`.  the field type argument may be passed
            as an instance of ``Type`` if type requires arguments or may simply
            be passed as a class. e.g::

                Field('id', MyType(foo='bar'))
                Field('id', MyType)

        :param source: Specify the source of this field. :py:class:`.Field` may specify
            a seperate name of the property or key to its `name`.
            When serializing kim will use the source attribute to perform a
            lookup on the data passed to field. e.g::

                obj.my_source = 'foo'
                Field('my_field', String, source='my_source')

                # this would generally map to
                {'my_field': 'foo'}

        :param field_id: Specify the ``field_id`` of this field.
            ``field_id`` is a unique name indentifying this field.  This
            can be used in conjunction with serializers to generate two fields
            dervied from the same name e.g::

                obj.foo = 'foo'
                obj.bar = 'bar'

                Field('my_name', String(), attr_name='name_1', source='foo')
                Field('my_name', String(), attr_name='name_2', source='bar')

                marshal(obj, role=whitelist('name_1'))
                {'my_name': 'foo'}

                marshal(obj, role=whitelist('name_2'))
                {'my_name': 'bar'}

        :param default: specify a default value of this `Field`.  Non required
            fields may specify a default value for a field when no data is
            provided during serialization.

        :param required: specify that data for this field is required when
            marshaling. During validation a fields source_value will be checked
            early on to make sure the field has a value when required.

        :param read_only: specify that this field should only be used when for
            output during serialization and MUST not accept a value during
            marshaling.  By default `Field` will simply ignore data for a read
            only data.

        :param allow_none: Allow the value of `Field` to be None when
            serializing.

        :param extra_validators: specify an iterable of callable validators
            that will be run along with `field_types` defined validate method.

            .. seealso::
                * :py:meth:`kim.fields.Field.is_valid`.
        """

        args = list(args)

        name = kwargs.pop('name', None)
        if args:
            if isinstance(args[0], basestring):
                if name is not None:
                    raise Exception(
                        "May not pass name positionally and as a keyword.")
                name = args.pop(0)

        field_type = kwargs.pop('field_type', None)
        if args:
            type_ = args[0]

            if hasattr(type_, '_kim_type'):
                if field_type is not None:
                    raise Exception(
                        "May not pass type_ positionally and as a keyword.")
                field_type = args.pop(0)

        if field_type is None or not hasattr(field_type, '_kim_type'):
            raise TypeError('field type is not a valid kim type.')
        elif inspect.isclass(field_type):
            field_type = field_type()

        self.field_type = field_type
        self.name = name
        self._source = kwargs.pop('source', None)
        self._field_id = kwargs.pop('field_id', None)
        self.default = kwargs.pop('default', None)
        self.required = kwargs.pop('required', True)
        self.allow_none = kwargs.pop('allow_none', True)
        self.read_only = kwargs.pop('read_only', False)
        self.extra_validators = kwargs.pop('extra_validators', [])

    @property
    def source(self):
        """``source`` is the attribute name the value of this field will be
        taken from when serializing. When marshaling, source is the name of
        the key that will be set on the resulting dict.

        if _source has no value then source will return ``self.name``

        :rtype: str
        :returns: the value of ``_source`` or ``name``
        """
        return self._source or self.name

    @property
    def field_id(self):
        """``field_id`` is a unique string identifying this Field.
        This can be passed to a role whitelist/blacklist.
        When instantiated via a Serializer, ``field_id`` will automatically be
        set to the attribute name of this Field on the serializer class.

        if _field_id has no value then _field_name will return ``self.name``

        :rtype: str
        :returns: the value of ``_field_id`` or ``name``
        """
        return self._field_id or self.name

    def marshal(self, value):
        """Call the :meth:`marshal_value` method of `type` providing the
        ``field_type`` validates.

        :returns: value returned from :meth:`marshal_value`
        """
        if self.is_valid(value):
            return self.field_type.marshal_value(value)

    def serialize(self, value):
        """Call the :meth:`serialize_value` method of `type`.

        :returns: value returned from :meth:`serialize_value`
        """
        return self.field_type.serialize_value(value)

    def is_valid(self, value):
        """validates a field against ``value``.  This method handles
        top level validation against field properties such
        as ``read_only`` and ``required``.

        Once field level validation is handled
        :py:meth:`kim.types.BaseType.validate` will be called.

        :param value: value passed to field for marshaling or serialization.

        :raises: :py:class:`kim.exceptions.ValidationError`
        :returns: True if all validators run without error.
        """

        # if read only is True, not further validation is needed.
        if self.read_only:
            return True

        if value is None:
            if self.required:
                raise ValidationError("This is a required field")
            elif not self.allow_none:
                raise ValidationError("This field can not be None")
            else:
                return True

        try:
            self.field_type.validate(value)
        except ValidationError as e:
            raise e

        for validator in self.extra_validators:
            try:
                validator(value)
            except ValidationError as e:
                raise e

        return True



#!/usr/bin/python
# -*- coding: utf-8 -*-

from collections import defaultdict
from datetime import date, datetime
import iso8601
import re
import decimal
import inspect
import importlib

from .exceptions import ValidationError, ConfigurationError
from .mapping import get_attribute, serialize, BaseMapping, marshal

from kim.utils import is_valid_type


def iskimtype(type_):
    return isinstance(type_, BaseType)


class BaseType(object):
    """:py:class:`.BaseType` defines the default ``Type`` api used by all
    kim types.
    """

    __visit_name__ = 'default'

    _kim_type = True

    error_message = 'An error ocurred validating this field'

    def get_error_message(self, value):
        """Return a valiation error message for this Type

        :returns: an error message explaining the error that occured
        """
        return unicode(self.error_message)

    def marshal_value(self, value):
        """called during marshaling of data.

        This method provides a hook for types to perform additonal operations
        on the ``value`` being marshaled.

        :rtype: mixed
        :returns: ``value``
        """

        return value

    def serialize_value(self, value):
        """called during serialization of data.

        This method provides a hook for types to perform additonal operations
        on the ``value`` being serialized.

        :rtype: mixed
        :returns: ``value``
        """

        return value

    def validate(self, value):
        """Validate that ``value`` is valid. If ``value``
        is invalid a :py:class:`kim.exceptions.ValidationError`
        will be raised e.g::

            def validate(self, value):
                if not isinstance(value, str):
                    raise ValidationError("Invalid type")

        :param value: the value being validated.

        :raises: :py:class:`kim.exceptions.ValidationError`
        :rtype: boolean
        :returns: True
        """

        return True


class TypedType(BaseType):
    """This type provides the base functionality for types that expect a
    specific python type e.g and :py:class:`.Integer` or :py:class:`.String`.
    """

    type_ = None

    error_message = 'This field was of an incorrect type'

    def __init__(self, *args, **kwargs):
        """Create a new :py:class:`.TypedType` constructed from ``args``
        and ``kwargs``.

        :param choices: provide a list of valid values for this type.  When
            validating this type the value will be checked to ensure its an
            allowed choice.

        """
        self.choices = kwargs.pop('choices', None)
        for k in kwargs:
            if k in ('name', 'field_type', 'source', 'field_id', 'default',
                     'required', 'read_only', 'allow_none'):
                raise ConfigurationError('Type %s was not expecting argument %s. ' \
                        'Did you mean to pass this to Field instead?' % (
                        self.__class__.__name__, k))
        super(TypedType, self).__init__(*args, **kwargs)

    def validate(self, value):
        """validates that ``value`` is of a given type

        .. seealso::
            * :py:meth:`kim.types.BaseType.validate`

        :raises: :class:`kim.exceptions.ValidationError`
        :returns: None
        """

        if value is not None:
            if not isinstance(value, self.type_):
                raise ValidationError(self.get_error_message(value))

            # TODO this should be moved to the ``Field`` api.
            if self.choices and value not in self.choices:
                raise ValidationError('not a valid choice')

        return True


class String(TypedType):
    """a ``TypedType`` that validate the incoming value is a valid ``str``
    """

    type_ = basestring


class Integer(TypedType):
    """a ``TypedType`` that validates the incoming value is a valid ``int``.
    """

    type_ = int


class Boolean(TypedType):
    """a ``TypedType`` that validates the incoming value is a valid ``bool``.
    """

    type_ = bool


class NumericType(BaseType):
    """a :py:class:`.TypedType` that allows integers to be passed as strings
    as well as standard ints.

    """

    type_ = int

    def __init__(self, *args, **kwargs):
        self.choices = kwargs.pop('choices', None)
        super(NumericType, self).__init__(*args, **kwargs)

    def validate(self, value):
        """validates that value is a valid int or can be converted to an int.

        .. seealso::
            :meth:`kim.types.BaseType.validate`

        :raises: :py:class:`kim.exceptions.ValidationError`
        :rtype: bool
        :returns: True
        """

        if value is not None:

            if (isinstance(value, basestring)
                    and not value.isdigit()):
                raise ValidationError(self.get_error_message(value))

            try:
                int(value)
            except ValueError:
                raise ValidationError(self.get_error_message(value))

            if self.choices and value not in self.choices:
                raise ValidationError('not a valid choice')

        return True


class PositiveInteger(Integer):
    """a ``TypedType`` that validates its value is an integer and that the value
    is not less than zero.
    """

    def validate(self, value):
        """validate that ``value`` is an integer and that it is not less
        than zero.

        :param value: value to be validated.
        """
        super(PositiveInteger, self).validate(value)
        if value is not None and value < 0:
            raise ValidationError('must be positive integer')

        return True


class Nested(BaseType):
    """Create a :py:class:`.Nested` mapping from a
    :py:class:`kim.mapping.BaseMapping` or from a mapped
    :py:class:`kim.serializers.Serializer` that allows users to create complex
    re-usable data structures in kim e.g::

        food = Mapping(Field('type', String()),
                       Field('name', String()))

        user_mapping = Mapping(Field('name', String()),
                               Field('foods', Nested(food_mapping))

    a :py:class:`.Nested` type may also specify a role to allow flexibly
    changing the types returned from a nested mapping.
    This further increases the flexibilty and reusability of mappings.
    For example, in certain cases when we want to re-use our food mapping
    in another mapping, we might not always want to return the
    'type' field. e.g::

        public_food_role = Role('public', 'name')
        user_mapping = Mapping(Field('name', String()),
                               Field('foods', Nested(food_mapping,
                                      role=public_food_role))

    In this example only the `name` field from the food mapping would be
    included in the returned data.

    .. seealso::

        * :py:class:`.BaseType`
        * :py:class:`kim.roles.Role`

    """

    __visit_name__ = 'nested'


    def __init__(self, mapped=None, role=None, *args, **kwargs):
        """:class:`Nested`

        :param name: name of this :py:class:`.Nested` type
        :param mapped: a :py:class:`kim.mapping.Mapping` or Mapped
            Serializer instance
        :param role: :py:class:`kim.roles.Role` role

        """

        self._mapping = None
        self.mapping = mapped
        self.role = role

        super(Nested, self).__init__(*args, **kwargs)

    @property
    def mapping(self):
        """Getter property to retrieve the mapping for this `Nested` type.

        :returns: self._mapping
        """
        # Allow mappings to be passed as a string if they don't exist yet
        # The setter has already done most of the work in getting the module,
        # but we need to actually resolve it here.
        if isinstance(self._mapping, str):
            self._mapping = getattr(self._mapping_module, self._mapping)

        if inspect.isclass(self._mapping):
            # Instantiate the class if not already
            self._mapping = self._mapping()

        try:
            mapping = self._mapping.__mapping__
        except AttributeError:
            mapping = self._mapping

        if not isinstance(mapping, BaseMapping):
            raise TypeError('Nested() must be called with a '
                            'mapping or a mapped serializer class or a mapped'
                            'serializer instance or a python path to one'
                            'of the above')
        return mapping

    @mapping.setter
    def mapping(self, mapped):
        """Setter for mapping property

        the :param:`mapped` arg must be a valid
        :class:`kim.mapping.BaseMapping` instance or any object exposing a
        __mapping__ attribute.

        :raises: TypeError
        """
        # Allow mappings to be passed as a string if they don't exist yet
        # see http://stackoverflow.com/questions/1095543/get-name-of-calling-functions-module-in-python
        # We store the module it came from here, but we don't actually resolve
        # the path until we need to access the mapping, because it won't be
        # in scope until then.
        if isinstance(mapped, str):
            if '.' in mapped:
                # A full python path has been passed
                module = '.'.join(mapped.split('.')[:-1])
                mapped = mapped.split('.')[-1]
                self._mapping_module = importlib.import_module(module)
            else:
                # Only a relative name has been passed, assume it's in
                # the same module who called us
                constructor_called_from = inspect.stack()[2]
                called_from_module = inspect.getmodule(constructor_called_from[0])
                self._mapping_module = called_from_module

        self._mapping = mapped

    def get_mapping(self):
        """Return the mapping defined for this `Nested` type.

        If a `role` has been passed to the `Nested` type the mapping
        will be run through the role automatically

        :returns: :class:`kim.mapping.BaseMapping` type

        .. seealso::
            :class:`kim.roles.Role`
        """
        if self.role:
            return self.role.get_mapping(self.mapping)

        return self.mapping

    def marshal_value(self, source_value):
        """marshal the `mapping` for this nested type

        :param source_value: data to marshal this `Nested` type to

        :returns: marshalled mapping
        """
        return marshal(self.get_mapping(), source_value)

    def serialize_value(self, source_value):
        """serialize `source_value` for this NestedType's mapping.

        :param source_value: data to serialize this `Nested` type to

        :returns: serialized mapping
        """
        return serialize(self.get_mapping(), source_value)

    def validate(self, source_value):
        """iterates Nested mapping calling validate for each
        field in the mapping.  Errors from each field will be stored
        and finally raised in a collection of errors

        :raises: ValidationError
        :returns: True
        """
        errors = defaultdict(list)

        for field in self.get_mapping().fields:
            value = get_attribute(source_value, field.name)
            try:
                field.is_valid(value)
            except ValidationError as e:
                errors[field.name].append(e.message)

        if errors:
            raise ValidationError(errors)
        else:
            return super(Nested, self).validate(source_value)


class Collection(TypedType):

    type_ = list

    __visit_name__ = 'collection'

    def __init__(self, inner_type, *args, **kwargs):
        self.inner_type = inner_type
        self.serialize_member = kwargs.pop('serialize_member', None)
        self.marshal_member = kwargs.pop('marshal_member', None)
        if not is_valid_type(self.inner_type):
            raise TypeError("Collection() requires a valid Type "
                            "as its first argument")
        super(Collection, self).__init__(*args, **kwargs)

    def serialize_members(self, source_value):
        if self.serialize_member:
            return [self.serialize_member(member) for member in source_value]
        else:
            if source_value is None:
                return []
            else:
                return source_value

    def marshal_members(self, source_value):
        if self.marshal_member:
            return [self.marshal_member(member) for member in source_value]
        else:
            if source_value is None:
                return []
            else:
                return source_value

    def marshal_value(self, source_value):

        return [self.inner_type.marshal_value(member)
                for member in self.marshal_members(source_value)]

    def serialize_value(self, source_value):

        return [self.inner_type.serialize_value(member)
                for member in self.serialize_members(source_value)]

    def validate(self, source_value):
        """Call :meth:`validate` on `type`.

        """
        super(Collection, self).validate(source_value)
        if source_value is not None:
            return [self.inner_type.validate(mem) for mem in source_value]


class DateTime(BaseType):
    type_ = datetime

    def serialize_value(self, source_value):
        if source_value is not None:
            return source_value.isoformat()

    def marshal_value(self, source_value):
        return iso8601.parse_date(source_value)

    def validate(self, source_value):
        super(DateTime, self).validate(source_value)
        if source_value is not None:
            try:
                iso8601.parse_date(source_value)
            except iso8601.ParseError:
                raise ValidationError('Date must be in iso8601 format')
        return True


class Date(DateTime):

    type_ = date

    def marshal_value(self, source_value):
        return super(Date, self).marshal_value(source_value).date()


class Regexp(String):

    def __init__(self, *args, **kwargs):
        self.pattern = kwargs.pop('pattern', re.compile('.*'))
        super(Regexp, self).__init__(*args, **kwargs)

    def validate(self, source_value):
        super(Regexp, self).validate(source_value)
        if source_value is not None and not self.pattern.match(source_value):
            raise ValidationError('Does not match regexp')
        return True


ipv4_re = re.compile(
    r'^(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}$')
validate_ipv4_address = Regexp(pattern=ipv4_re).validate


# TODO handle validation ipv6 domains.
def validate_ipv46_address(value):
    try:
        validate_ipv4_address(value)
    except ValidationError as e:
        raise e


class Email(String):
    # Django from django forms

    message = 'Enter a valid email address.'

    user_regex = re.compile(
        r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*$"  # dot-atom
        r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"$)',  # quoted-string
        re.IGNORECASE)
    domain_regex = re.compile(
        r'(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}|[A-Z0-9-]{2,})$',
        re.IGNORECASE)
    literal_regex = re.compile(
        # literal form, ipv4 or ipv6 address (SMTP 4.1.3)
        r'\[([A-f0-9:\.]+)\]$',
        re.IGNORECASE)

    domain_whitelist = ['localhost']

    def __init__(self, message=None, whitelist=None, *args, **kwargs):
        super(Email, self).__init__(*args, **kwargs)
        if message is not None:
            self.message = message
        if whitelist is not None:
            self.domain_whitelist = whitelist

    def validate(self, source_value):

        if not source_value or '@' not in source_value:
            raise ValidationError(self.message)

        user_part, domain_part = source_value.rsplit('@', 1)

        if not self.user_regex.match(user_part):
            raise ValidationError(self.message)

        if (domain_part not in self.domain_whitelist and
                not self.validate_domain_part(domain_part)):
            # Try for possible IDN domain-part
            try:
                domain_part = domain_part.encode('idna').decode('ascii')
                if self.validate_domain_part(domain_part):
                    return True
            except UnicodeError:
                pass
            raise ValidationError(self.message)

        return True

    def validate_domain_part(self, domain_part):
        if self.domain_regex.match(domain_part):
            return True

        literal_match = self.literal_regex.match(domain_part)
        if literal_match:
            ip_address = literal_match.group(1)
            try:
                validate_ipv46_address(ip_address)
                return True
            except ValidationError:
                pass
        return False


class Float(BaseType):

    def __init__(self, *args, **kwargs):
        self.as_string = kwargs.pop('as_string', False)
        super(Float, self).__init__(*args, **kwargs)

    def validate(self, source_value):
        super(Float, self).validate(source_value)
        if source_value is not None:
            if self.as_string:
                if not isinstance(source_value, str):
                    raise ValidationError(self.get_error_message(source_value))

                # Now just check we can cast it to a float
                try:
                    float(source_value)
                except ValueError:
                    raise ValidationError('Not a valid float')
            else:
                if not isinstance(source_value, float):
                    raise ValidationError(self.get_error_message(source_value))
        return True

    def serialize_value(self, source_value):
        if self.as_string:
            return str(source_value)
        else:
            return source_value

    def marshal_value(self, source_value):
        return float(source_value)


class Decimal(BaseType):
    type_ = decimal.Decimal

    def __init__(self, *args, **kwargs):
        decimals = kwargs.pop('precision', 5)
        self.precision = decimal.Decimal('0.' + '0' * (decimals - 1) + '1')
        super(Decimal, self).__init__(*args, **kwargs)

    def _cast(self, value):
        return decimal.Decimal(value).quantize(self.precision)

    def validate(self, source_value):
        super(Decimal, self).validate(source_value)
        if source_value is not None:
            if not isinstance(source_value, str):
                raise ValidationError(self.get_error_message(source_value))

            # Now just check we can cast it to a Decimal
            try:
                self._cast(source_value)
            except decimal.InvalidOperation:
                raise ValidationError('Not a valid decimal')

        return True

    def serialize_value(self, source_value):
        return str(self._cast(source_value))

    def marshal_value(self, source_value):
        return self._cast(source_value)
